﻿using System;
using System.Collections.Generic;
using System.Text;
using IDTechSDK;

namespace SDKuDemo
{
    class Receipts
    {
        public static int printerWidth = 30;

        private static string printCenter(String str)
        {
            if (str == null) str = "";
            int len = str.Length;
            if (len > printerWidth-2)
            {
                return str;
            }
            for (int x = 0; x < (printerWidth - len) / 2; x++)
            {
                str = " " + str;
            }
            return str ;
        }
        private static string printLine(char character)
        {
            string str = "";
            for (int x = 0; x < printerWidth; x++) str += character;
            return str ;
        }

        private static string printJustified(String str, String str2)
        {
            if (str == null) str = "";
            if (str2 == null) str2 = "";
            int len = str.Length + str2.Length;
            if (len > printerWidth - 1)
            {
                return str + str2;
            }
            for (int x = len; x < printerWidth; x++)
            {
                str2 = " " + str2;
            }
            return str + str2 ;
        }

        public static string companyName = "Joe's Restaurant";
        public static string companyAddress = "10721 Walker Street";
        public static string companyCityStateZip = "Cypress, CA 90630";
        public static string companyPhone = "714-761-8886";
        public static string companyURL= "www.joesrestaurant.net";




        public static string getReceipt(IDTTransactionData card, string _approvalCode = "135812", string _msrAmount = "1.00", bool hostApproved = false)
        {
            //text receipt set for 30 characters per line

            Dictionary<string, string> dict = IDTechSDK.Common.processTLVUnencrypted(card.emv_unencryptedTags);
            Dictionary<string, string> maskedTags = IDTechSDK.Common.processTLVUnencrypted(card.emv_maskedTags);
            String ff8105 = "";
            dict.TryGetValue("FF8105", out ff8105);
            if (ff8105 != null && ff8105.Length > 0)
            {
                Common.combineTLV(ref card.emv_unencryptedTags, Common.getByteArray(ff8105));
                dict = IDTechSDK.Common.processTLVUnencrypted(card.emv_unencryptedTags);

            }
            dict.TryGetValue("FF8106", out ff8105);
            if (ff8105 != null && ff8105.Length > 0)
            {
                Common.combineTLV(ref card.emv_unencryptedTags, Common.getByteArray(ff8105));
                dict = IDTechSDK.Common.processTLVUnencrypted(card.emv_unencryptedTags);

            }
            StringBuilder rec = new StringBuilder() ;
            rec.AppendLine(printLine('='));
            rec.AppendLine(printCenter("RECEIPT"));

            rec.AppendLine(printLine('='));
            string dateString = DateTime.Now.ToString("yyyy/MM/dd HHmmss");
            rec.AppendLine(printCenter(companyName));

            rec.AppendLine(printCenter(companyAddress));
            rec.AppendLine(printCenter(companyCityStateZip));
            rec.AppendLine(printCenter(companyPhone));
            rec.AppendLine(printCenter(companyURL));

            rec.AppendLine(printLine('-'));



            if (card.emv_unencryptedTags == null && (card.msr_track1 != null || card.msr_track2 != null))
            {

                rec.AppendLine(printJustified("Entry Method: ", "SWIPED"));


                string _pan2 = card.msr_track2;
                if (_pan2.Length > 0)
                {
                    string[] ar = _pan2.Split('=');
                    _pan2 = ar[0];
                }

                string _name2 = "";

                if (_name2 == null && card.msr_track1 != null && card.msr_track1.Length > 0)
                {
                    char[] splits = { '^' };
                    string[] strs = card.msr_track1.Split(splits);
                    if (strs.Length > 1) _name2 = strs[1];
                }

                rec.AppendLine(printLine('-'));
                if (_pan2.Length > 6)
                {
                    rec.AppendLine(printJustified("Account:", "************" + _pan2.Substring(_pan2.Length - 4, 4)));

                }
                else
                {
                    rec.AppendLine(printJustified("Account:", _pan2));

                }
                rec.AppendLine();
                rec.AppendLine(printJustified("Date/Time:", dateString));



                rec.AppendLine(printJustified("Amount:", _msrAmount));


                rec.AppendLine(printJustified("Auth Code:", _approvalCode));
                return rec.ToString();
            }
            string swiped = "";
            bool isGood = false;
            string reason = "";
            switch (card.emv_resultCode)
            {
                case EMV_RESULT_CODE.EMV_RESULT_CODE_UNKNOWN:
                    reason = "Reason: ONLINE APPROVAL";
                    isGood = true;
                    break;
                case EMV_RESULT_CODE.EMV_RESULT_CODE_APPROVED_OFFLINE:
                    reason = "Reason: OFFLINE APPROVAL";
                    isGood = true;
                    break;
                case EMV_RESULT_CODE.EMV_RESULT_CODE_DECLINED_OFFLINE:
                    reason = "Reason: OFFLINE DECLINE";

                    break;
                case EMV_RESULT_CODE.EMV_RESULT_CODE_APPROVED:
                    reason = "Reason: ONLINE APPROVAL";
                    isGood = true;
                    break;
                case EMV_RESULT_CODE.EMV_RESULT_CODE_MSR_SWIPE_CAPTURED:
                    reason = "Reason: ONLINE APPROVAL";
                    isGood = true;
                    break;
                case EMV_RESULT_CODE.EMV_RESULT_CODE_FALLBACK_TO_MSR:
                    reason = "Reason: ONLINE APPROVAL";
                    isGood = true;
                    break;
                case EMV_RESULT_CODE.EMV_RESULT_CODE_DECLINED:
                    reason = "Reason: OFFLINE DECLINE";
                    break;

                case EMV_RESULT_CODE.EMV_RESULT_CODE_CALL_YOUR_BANK:
                    reason = "Reason: CALL BANK";
                    break;
                case EMV_RESULT_CODE.EMV_RESULT_CODE_NOT_ACCEPTED:
                    reason = "Reason: NOT ACCEPTED";
                    break;
   

            }
            if (!isGood && hostApproved) isGood = true;
            if (isGood)
            {
                rec.AppendLine(printCenter("TRANSACTION SUCCESSFUL"));
            }
            else
            {
                rec.AppendLine(printCenter("TRANSACTION FAILED"));
            }
            rec.AppendLine(printCenter(reason));
            IDTTransactionData card2 = null;
            IDTechSDK.IDT_Device.sharedController.emv_retrieveTransactionResult(new byte[] { 0x9F, 0x16, 0x5A, 0x5F, 0x20,0xff,0xee,0x05,0x9C, 0x91, 0x9F, 0x06,0x9F, 0x02, 0x8A, 0x9f, 0x26 }, ref card2);
            Dictionary<string, string> dict2 = null;
            Dictionary<string, string> maskedTags2 = null;

            if (card2 != null && card2.emv_unencryptedTags != null) dict2 = IDTechSDK.Common.processTLVUnencrypted(card2.emv_unencryptedTags);
            if (card2 != null && card2.emv_maskedTags != null)  maskedTags2 = IDTechSDK.Common.processTLVUnencrypted(card2.emv_maskedTags);

            if (dict2 == null) dict2 = new Dictionary<string, string>();
            if (maskedTags2 == null) maskedTags2 = new Dictionary<string, string>();
            String MID = "";
            dict.TryGetValue("9F16", out MID);
            if (MID == null || MID.Length == 0) dict2.TryGetValue("9F16", out MID); ;
            if (MID == null || MID.Length == 0) MID = "";
            byte[] bMID = IDTechSDK.Common.getByteArray(MID);
            String MIDStr = IDTechSDK.Common.byteToString(bMID, 0, bMID.Length);

            string method = "";
           

            switch (card.captureCardType)
            {
                case CAPTURE_CARD_TYPE.CAPTURE_CARD_TYPE_UNKNOWN:
                    break;
                case CAPTURE_CARD_TYPE.CAPTURE_CARD_TYPE_CONTACT:
                    method = "CONTACT";
                    break;
                case CAPTURE_CARD_TYPE.CAPTURE_CARD_TYPE_CTLS_EMV:
                    method = "CTLS";

                    break;
                case CAPTURE_CARD_TYPE.CAPTURE_CARD_TYPE_CTLS_MSD:
                    method = "MSD";

                    break;
                case CAPTURE_CARD_TYPE.CAPTURE_CARD_TYPE_MSR:
                    method = "MSR";

                    break;
                default:
                    break;
            }

            string _pan = "";
            if (!dict.TryGetValue("5A", out _pan))
            {
                if (!maskedTags.TryGetValue("5A", out _pan))
                {
                    if (!dict2.TryGetValue("5A", out _pan))
                    {
                        
                            maskedTags2.TryGetValue("5A", out _pan);
                        

                    }
                }
                
            }
            if (card.captureCardType == CAPTURE_CARD_TYPE.CAPTURE_CARD_TYPE_UNKNOWN)
            {

                rec.AppendLine(printJustified("Entry Method: ", "UNKNOWN"));
                
               
            }
            else
            {
                rec.AppendLine(printJustified("Entry Method: ", method));
            }





            rec.AppendLine();
            if (card.msr_track2Length > 0) _pan = card.msr_track2;
            if (_pan == null) _pan = "";

            if (_pan.Length > 0)
            {
                string[] ar = _pan.Split('=');
                _pan = ar[0];
            }

            string _name = "";
            dict.TryGetValue("5F20", out _name);
            if (_name == null || _name.Length == 0) dict2.TryGetValue("5F20", out _name);
            if (_name != null)
            {
                if (_name.Length > 0)
                    _name = Common.getASCIIArray(_name);
            }
            else
            {
                if (!dict.TryGetValue("FFEE05", out _name)) dict2.TryGetValue("FFEE05", out _name);
                if (_name != null)
                {
                    if (_name.Length > 0)
                    {
                        byte[] ffee05 = Common.getByteArray(_name);
                        Dictionary<string,string> dict3 = IDTechSDK.Common.processTLVUnencrypted(ffee05);
                        dict3.TryGetValue("5F20", out _name);
                        if (_name != null)
                        {
                            if (_name.Length > 0)
                                _name = Common.getASCIIArray(_name);
                        }
                    }

                }
            }

            string t1 = "";
            dict.TryGetValue("56", out t1);
            if (t1 != null && t1.Length > 0) card.msr_track1 = Common.getASCIIArray(t1);


            if (_name == null && card.msr_track1 != null && card.msr_track1.Length > 0)
            {
                char[] splits = { '^' };
                string[] strs = card.msr_track1.Split(splits);
                if (strs.Length > 1) {
                    _name = strs[1];
                    _pan = strs[0];
                }
                
            }
            string _aid = "";
           if (! dict.TryGetValue("9F06", out _aid)) dict2.TryGetValue("9F06", out _aid); ;
            if (_aid == null) if (!dict.TryGetValue("84", out _aid)) dict2.TryGetValue("84", out _aid);
            string _transType = "";
            if (!dict.TryGetValue("9C", out _transType)) dict2.TryGetValue("9C", out _transType);
            if (_transType == null) _transType = "";
            string _iad = "";
            if (!dict.TryGetValue("91", out _iad)) dict2.TryGetValue("91", out _iad);
            if (_iad == null) _iad = "";
            string _amt = "";

            if (_aid == null) _aid = "";


            if (!dict.TryGetValue("9F02", out _amt)) dict2.TryGetValue("9F02", out _amt);
            if (_amt == null) _amt = "";
            rec.AppendLine(printLine('-'));
          if (_name != null && _name.Length > 0)  rec.AppendLine(printJustified("Name:", _name));

            if (_pan.Length > 6)
            {
                rec.AppendLine(printJustified("Account:", "************" + _pan.Substring(_pan.Length - 4, 4)));

            }
            else
            {
                rec.AppendLine(printJustified("Account:", _pan));

            }
            if (_transType.Length == 0) _transType = swiped;
            rec.AppendLine(printJustified("Transaction Type:", _transType));
            rec.AppendLine(printJustified("Date/Time:", dateString));



            if (_amt.Length == 0 || _amt.Equals("000000000000")) _amt = _msrAmount;
            if (_amt.Length > 0) rec.AppendLine(printJustified("Amount:", _amt));
            if (card.msr_track2Length == 0 && _iad.Length != null && _iad.Length > 0)

                rec.AppendLine(printJustified("IAD:", _iad));


            if (_aid.Length > 0) rec.AppendLine(printJustified("AID:", _aid));


            string authcode = "";
            if (!dict.TryGetValue("8A", out authcode))
            {
                dict2.TryGetValue("8A", out authcode);
            }
            if (authcode == null || authcode.Length == 0)
            {
                if (_approvalCode != null && _approvalCode.Length > 0) authcode = _approvalCode;
                else authcode = "1245223";
            }
            else
            {
                byte[] auth = IDTechSDK.Common.getByteArray(authcode);
                authcode = IDTechSDK.Common.byteToString(auth, 0, auth.Length);
            }

            rec.AppendLine(printJustified("Auth Code:", authcode));
            string aac = "";
           if (! dict.TryGetValue("9F26", out aac)) dict2.TryGetValue("9F26", out aac);
           
            if (card.msr_track2Length == 0 && aac != null && aac.Length > 0)
            {
                rec.AppendLine(printJustified("TC:", aac));
            }


            if (_name == null) _name = "";


            return rec.ToString();


        }

    }
}
